package com.edtoon.rott;

import com.edtoon.rott.ott.OTTReader;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.File;
import java.io.IOException;

/*
 * Command line RTX/RTTL->OTT parser/converter.
 */
public class Main
{
	public static void printSyntax()
	{
		System.out.println("Syntax: " + Main.class.getName() + " [-parse file] / [-convert inputfile outputfile]" +
			"/ [-decode inputfile]");
	}

	public static void printFileTypes()
	{
		System.out.println("Possible file types: .rtx, .rtttl, .txt");
	}

	public static void main(final String[] args)
	{
		boolean convert = false, rtx = false, ott = false;
		ToneParser parser = null;
		OTTReader reader = null;
		String fileName = null;
		byte[] contents = null;
		File file = null;
		int parseResult;

		if(args == null || args.length == 0)
		{
			printSyntax();
			System.exit(0);
		}

		try {
			file = findFile(args[1]);

			if(args.length == 2 && args[0].equals("-parse"))
			{
			} else if(args.length == 3 && args[0].equals("-convert")) {
				convert = true;
			} else if(args.length == 2 && args[0].equals("-decode")) {
				ott = true;
			} else {
				printSyntax();
				System.exit(Errors.ERROR_COMMAND_NOT_RECOGNIZED);
			}
		} catch(final FileNotFoundException fnfe) {
			fnfe.printStackTrace(System.err);
			System.exit(Errors.ERROR_INPUT_FILE_NOT_FOUND);
		}

		fileName = file.getName();

		if(fileName.endsWith(".rtttl") || fileName.endsWith(".txt"))
		{
			System.out.println("Reading RTTTL file...");
		} else if(fileName.endsWith(".rtx")) {
			System.out.println("Reading RTX file...");
		} else if(fileName.endsWith(".ott") && ott) {
			System.out.println("Reading OTT file...");
		} else {
			printFileTypes();
			printSyntax();
			System.exit(Errors.ERROR_FILE_TYPE_NOT_RECOGNIZED);
		}

		try {
			contents = IOUtil.getFileContents(file);

			if(ott)
			{
				reader = new OTTReader();
				reader.read(contents);
				return;
			}

			parser = new ToneParser();
			parser.setDoOutput(!convert);

			if(convert)
			{
				FileOutputStream out = new FileOutputStream(args[2]);
				parser.parse(contents, out);
				out.close();
				System.out.println(args[2] + " written.");
			} else {
				parser.parse(contents);
			}

			System.exit(0);
		} catch(final IOException ioe) {
			ioe.printStackTrace(System.err);
			System.exit(Errors.ERROR_FILE_READ_ERROR);
		} catch(final ToneParser.ToneParseException rpe) {
			rpe.printStackTrace(System.err);
			System.exit(rpe.getErrorNumber());
		} catch(final OTTReader.OTTReaderException otre) {
			otre.printStackTrace(System.err);
			System.exit(1);
		}
	}

	private static File findFile(final String fileName)
		throws FileNotFoundException
	{
		File file = new File(fileName);

		if(!file.exists() || !file.isFile())
		{
			throw new FileNotFoundException("File not found - " + fileName);
		}

		if(!file.canRead())
		{
			throw new FileNotFoundException("File could not be read - " + fileName);
		}

		return file;
	}
}
