package com.edtoon.rott.ott;

import java.util.HashMap;

public class OTTReader
{
	private static final boolean DEBUG = true;
	public OTTReader() { }

	public void read(final byte[] bytes)
		throws OTTReaderException
	{
		StringBuffer buf = new StringBuffer();
		String binary = null, tmp = null;
		int position = 0;

		for(int i = 0; i < bytes.length; i++)
		{
			byte b = bytes[i];

			for(int j=7; j >=0; j--)
			{
				buf.append((b >> j) & 1);
			}
		}

		binary = buf.toString();

		tmp = binary.substring(position, 8);
		int commandLength = Integer.parseInt(tmp, 2);
		if(commandLength != 2)
		{
			throw new OTTReaderException("Invalid command length. Should be two, was: " + commandLength);
		}
		debug("Command-length: " + commandLength);
		position += 8;

		tmp = binary.substring(position, position + 8);
		if(!tmp.equals("01001010"))
		{
			throw new OTTReaderException("Ringing tone programming exception not found.");
		}
		debug("Ringing tone programming.");
		position += 8;

		tmp = binary.substring(position, position + 7);
		if(!tmp.equals("0011101"))
		{
			throw new OTTReaderException("Command part invalid for a sound.");
		}
		debug("Sound");
		position += 7;

		tmp = binary.substring(position, position + 3);
		if(!tmp.equals("001"))
		{
			throw new OTTReaderException("Song type is not basic.");
		}
		debug("Basic song type");
		position += 3;

		tmp = binary.substring(position, position + 4);
		int titleLength = Integer.parseInt(tmp, 2);
		debug("Title length: " + titleLength);
		position += 4;

		buf = new StringBuffer();
		for(int i = 0; i < titleLength; i++)
		{
			buf.append((char)Integer.parseInt(binary.substring(position, position + 8), 2));
			position += 8;
		}
		debug("Title: " + buf.toString());

		tmp = binary.substring(position, position + 8);
		if(!tmp.equals("00000001"))
		{
			throw new OTTReaderException("Song sequence length not 1.");
		}
		debug("Song sequence length: 1");
		position += 8;

		int patternLength = 1;

		for(int i = 0; i < patternLength; i++)
		{
			String instruction = binary.substring(position, position + 3);
			position += 3;

			if(instruction.equals(OTTConstants.TEMPO_INSTRUCTION_ID))
			{
				debug("[" + (i+1) + "] Tempo instruction: " + describeTempo(
					binary.substring(position, position + 5)));
				position += 5;
			} else if(instruction.equals(OTTConstants.SCALE_INSTRUCTION_ID)) {
				debug("[" + (i+1) + "] Scale instruction: " + describeScale(
					binary.substring(position, position + 2)));
				position += 2;
			} else if(instruction.equals(OTTConstants.STYLE_INSTRUCTION_ID)) {
				debug("[" + (i+1) + "] Style instruction: " + describeStyle(
					binary.substring(position, position + 2)));
				position += 2;
			} else if(instruction.equals(OTTConstants.VOLUME_INSTRUCTION_ID)) {
				debug("[" + (i+1) + "] Volume instruction: " + describeVolume(
					binary.substring(position, position + 4)));
				position += 4;
			} else if(instruction.equals(OTTConstants.NOTE_INSTRUCTION_ID)) {
				debug("[" + (i+1) + "] Note: " + describeNote(
					binary.substring(position, position + 9)));
				position += 9;
			} else if(instruction.equals(OTTConstants.PATTERN_HEADER_ID)) {
				debug("Pattern header ID");
				patternLength = describePattern((i+1), binary.substring(position, position + 14));
				position += 14;
				i = 0;
			} else {
				throw new OTTReaderException("UNKNOWN INSTRUCTION! Data: " + instruction);
			}
		}

		int mod = position % 8;

		if(mod != 0)
		{
			int remaining = 8-mod;
			tmp = binary.substring(position, position + remaining);
			debug("Padding to close: " + tmp);
			position += remaining;
		} else {
			debug("NOT PADDING TO CLOSE");
		}

		tmp = binary.substring(position, position + 8);
		if(!tmp.equals("00000000"))
		{
			throw new OTTReaderException("Final command was not end");
		}
		debug("Command end");
		position += 8;

		if(position != binary.length())
		{
			throw new OTTReaderException("" + (binary.length()-position) + " characters remain unaccounted for.");
		}
	}

	public int describePattern(final int position, final String str)
		throws OTTReaderException
	{
		String part = (String)OTTConstants.partDesc.get(str.substring(0, 2));
		int loopValue = Integer.parseInt(str.substring(2, 6), 2),
			patternLength = Integer.parseInt(str.substring(6, 14), 2);

		debug("[" + position + "] Pattern Header: " + part + "\nLoop value: " + loopValue +
			"\nPattern length: " + patternLength);

		return patternLength;
	}

	public String describeNote(final String str)
		throws OTTReaderException
	{
		String note = (String)OTTConstants.noteDesc.get(str.substring(0, 4)),
			duration = (String)OTTConstants.durationDesc.get(str.substring(4, 7)),
			special = (String)OTTConstants.specialDesc.get(str.substring(7, 9));

		if(note == null)
		{
			throw new OTTReaderException("Note setting invalid");
		}

		if(duration == null)
		{
			throw new OTTReaderException("Duration setting invalid");
		}

		if(special == null)
		{
			throw new OTTReaderException("Special duration setting invalid");
		}

		return note + ", duration: " + duration + ", special: " + special;
	}

	public String describeVolume(final String str)
		throws OTTReaderException
	{
		String value = (String)OTTConstants.volumeDesc.get(str);

		if(str == null)
		{
			throw new OTTReaderException("Volume setting invalid");
		}

		return value;
	}

	public String describeStyle(final String str)
		throws OTTReaderException
	{
		String value = (String)OTTConstants.styleDesc.get(str);

		if(str == null)
		{
			throw new OTTReaderException("Style setting invalid");
		}

		return value;
	}

	public String describeScale(final String str)
		throws OTTReaderException
	{
		String value = (String)OTTConstants.scaleDesc.get(str);

		if(str == null)
		{
			throw new OTTReaderException("Scale setting invalid");
		}

		return value;
	}

	public String describeTempo(final String str)
		throws OTTReaderException
	{
		String value = (String)OTTConstants.tempoDesc.get(str);

		if(str == null)
		{
			throw new OTTReaderException("Tempo setting invalid");
		}

		return value;
	}

	public class OTTReaderException extends Exception
	{
		public OTTReaderException(final String msg)
		{
			super(msg);
		}
	}

	private static void debug(final String str)
	{
		if(DEBUG)
		{
			System.out.println(str);
		}
	}
}
